/*
 * Copyright (C) 2007 Texas Instruments Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
	ROM area
	SYNC  MAC_ADDR DUMMY
	1Byte 6Byte    1Byte
	H/W Ver
	2Byte
*/

#include <common.h>
#include <asm/arch/sys_proto.h>

#define SPICMD_EWEN_OPCODE		0x00
#define SPICMD_ERASE_OPCODE		0x03
#define SPICMD_EWDS_OPCODE		0x00

#define SPICMD_EWEN_ADDR		0xFF
#define SPICMD_EWDS_ADDR		0x00

#define GPIO_CS					19		//# output
#define GPIO_SCLK				18		//# output
#define GPIO_SDI				17		//# input
#define GPIO_SDO				16		//# output

#define AT93C66_CS_ACTIVE		1
#define AT93C66_CS_INACTIVE		0
#define AT93C66_CMD_BITS		12
#define AT93C66_DATA_BITS		8

#define EEPROM_SIZE				128	//8bit
#define AT93C66_EEPROM_READ		0xC000	/* read */
#define AT93C66_EEPROM_WRITE	0xA000	/* write */
#define AT93C66_EEPROM_WREN		0x9FF0	/* write enable */
#define AT93C66_EEPROM_ERASE	0xE000	/* location erase 1byte */
#define AT93C66_EEPROM_ERAL		0x9000	/* location all */

#define mdelay(n)				udelay((n)*1000)
#define do_spidelay(n)			udelay(n)

//---------------------------------------
#define ROM_MAC_SYNC	0x4E		//# 'N' for mac is valid
#define ROM_MAC_ADDR0	0
#define ROM_MAC_SIZE	8
#define ROM_MAC_ADDR1	(ROM_MAC_ADDR0+ROM_MAC_SIZE)

#define ROM_HW_ADDR		(ROM_MAC_ADDR1+ROM_MAC_SIZE)
#define ROM_HW_SIZE		2

static void at93c66_send_cmd(unsigned short word)
{
	int i;

	gpio_write_data(GPIO_CS, AT93C66_CS_ACTIVE);
	do_spidelay(1); 	/* min. setup time 200ns */

	for(i = 0; i < AT93C66_CMD_BITS; i++)
	{
		/* setup MSB (to slave) on leading edge */
		gpio_write_data(GPIO_SCLK, 0);
		gpio_write_data(GPIO_SDO, word & (1 << 15));
		do_spidelay(2); 	/* T (setup) 100ns */

		word <<=1;
		gpio_write_data(GPIO_SCLK, 1);
		do_spidelay(2);
	}
}

static unsigned char at93c66_read_byte(int addr)
{
	unsigned short word;
	int i;
	unsigned char data = 0;

	word = (addr & 0x1FF) << 4;
	word |= AT93C66_EEPROM_READ;

	at93c66_send_cmd(word);

	// Note. Dummy bit discard the 8 bit data outputs.
	gpio_write_data(GPIO_SCLK, 0);
	gpio_read_data(GPIO_SDI);
	do_spidelay(2);
	gpio_write_data(GPIO_SCLK, 1);
	do_spidelay(2);

	for(i = 0; i < AT93C66_DATA_BITS; i++) {
		data <<= 1;
		/* sample MSB (from slave) on trailing edge */
		gpio_write_data(GPIO_SCLK, 0);
		data |= gpio_read_data(GPIO_SDI);
		do_spidelay(2); 	/* T(setup) */

		gpio_write_data(GPIO_SCLK, 1);
		do_spidelay(2);
	}

	gpio_write_data(GPIO_CS, AT93C66_CS_INACTIVE);

	return data;
}

static void at93c66_write_enable(void)
{
	unsigned short word;

	word = AT93C66_EEPROM_WREN;

	at93c66_send_cmd(word);

	gpio_write_data(GPIO_CS, AT93C66_CS_INACTIVE);

	do_spidelay(2);
}

static void at93c66_erase_page(int addr)
{
	unsigned short word;

	word = (addr & 0x1FF) << 4;
	word |= AT93C66_EEPROM_ERASE;

	at93c66_send_cmd(word);

	gpio_write_data(GPIO_CS, AT93C66_CS_INACTIVE);

	do_spidelay(2);

	// wait max 10ms..
	mdelay(20);
}

static void at93c66_erase_all(void)
{
	unsigned short word;

	word = AT93C66_EEPROM_ERAL;

	at93c66_send_cmd(word);

	gpio_write_data(GPIO_CS, AT93C66_CS_INACTIVE);

	do_spidelay(2);

	// wait max 10ms..
	mdelay(20);
}

static void at93c66_write_byte(unsigned char data, int addr)
{
	unsigned short word;
	int i;

	at93c66_write_enable();

	at93c66_erase_page(addr);

	word = (addr & 0x1FF) << 4;
	word |= AT93C66_EEPROM_WRITE;

	at93c66_send_cmd(word);

	for(i = 0; i < AT93C66_DATA_BITS; i++) {
		/* sample MSB (from slave) on trailing edge */
		gpio_write_data(GPIO_SCLK, 0);
		gpio_write_data(GPIO_SDO, data & 0x80);
		do_spidelay(2); 	/* T(setup) */

		data <<= 1;
		gpio_write_data(GPIO_SCLK, 1);
		do_spidelay(2);
	}

	gpio_write_data(GPIO_CS, AT93C66_CS_INACTIVE);

	do_spidelay(2);

	// wait max 10ms..
	mdelay(20);
}


static int at93c66_eeprom_read(char *buf, unsigned int count, unsigned int pos)
{
	char *p;
	unsigned int i;

	p = (char *)buf;

	if (pos >= EEPROM_SIZE)
		return 0;

	for (i = pos; count > 0 && i < EEPROM_SIZE; ++i, ++p, --count) {
		*p = at93c66_read_byte(i);
	}

	return (p-buf);
}

static int at93c66_eeprom_write(const char *buf, unsigned int count, unsigned int pos)
{
	unsigned int i;
	unsigned char c;
	const char *p;

	p = (const char *)buf;

	if (pos >= EEPROM_SIZE)
		return 0;

	for (i = pos; count > 0 && i < EEPROM_SIZE; ++i, ++p, --count) {
		c = *p;
		at93c66_write_byte(c, i);
	}

	return (p-buf);
}

int dvr_eeprom_read(unsigned char *buf, int id)
{
	char tmp[ROM_MAC_SIZE];
	int addr;

	if(id==0 || id==1)		//# read mac addr
	{
		if(id==1)
			addr = ROM_MAC_ADDR1;
		else
			addr = ROM_MAC_ADDR0;

		/* Set Ethernet MAC address from SPI EEPROM */
		if (!at93c66_eeprom_read((char *)tmp, ROM_MAC_SIZE, addr)) {
			printf("\nEEPROM read FAILED!!!\n");
		} else {
			if (tmp[0] == ROM_MAC_SYNC) {
				sprintf(buf, "%02x:%02x:%02x:%02x:%02x:%02x",
						tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], tmp[6]);
				return 1;
			}
		}
	}
	else if(id==2)			//# read hw version
	{
		addr = ROM_HW_ADDR;

		if (!at93c66_eeprom_read((char *)tmp, ROM_HW_SIZE, addr)) {
			printf("\nEEPROM read FAILED!!!\n");
		} else {
			sprintf(buf, "%02x:%02x", tmp[0], tmp[1]);
			return 1;
		}
	}

	return 0;
}

int dvr_netra_eeprom_init(void)
{
	//# enable GPIO for SPI - default '0'
	gpio_set_oe(GPIO_CS, 1);
	gpio_set_oe(GPIO_SCLK, 1);
	gpio_set_oe(GPIO_SDI, 0);
	gpio_set_oe(GPIO_SDO, 1);

	return 0;
}

int dvr_get_hw_version(void)
{
	unsigned char buf[ROM_HW_SIZE];
	int dvr_hwver=0;

	at93c66_eeprom_read((char *)buf, ROM_HW_SIZE, ROM_HW_ADDR);
	dvr_hwver = (buf[0]<<8) + buf[1];

	return dvr_hwver;
}

/******************************************************************************
 * DVR Board specific command saving to eeprom
 *****************************************************************************/
static int do_save_dvrrom (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	int i;
	u_int8_t env_ethaddr[ROM_MAC_SIZE];
	u_int8_t env_ver[2];
	char *s = NULL,*end;

	/* at least two arguments please */
	if (argc < 2)
		goto usage;

	if (strcmp (argv[1], "save") == 0)
	{
		/* Read Ethernet MAC address from the U-Boot environment. */
		s = getenv("ethaddr");
		if(s == NULL) {
			printf ("No valid MAC address0 in U-boot environment\n");
		} else {
			env_ethaddr[0] = ROM_MAC_SYNC;
			for (i = 1; i < ROM_MAC_SIZE-1; i++) {
				env_ethaddr[i] = s ? simple_strtoul(s, &end, 16) : 0;
				if (s)
					s = (*end) ? end+1 : end;
			}
			env_ethaddr[7] = 0;

			at93c66_eeprom_write((const char *)env_ethaddr, ROM_MAC_SIZE, ROM_MAC_ADDR0);
			printf("Set MAC address0 from environment\n");
		}

		/* Read Ethernet1 MAC address from the U-Boot environment. */
		s = getenv("eth1addr");
		if(s == NULL) {
			printf ("No valid MAC address1 in U-boot environment\n");
		} else {
			env_ethaddr[0] = ROM_MAC_SYNC;
			for (i = 1; i < ROM_MAC_SIZE-1; i++) {
				env_ethaddr[i] = s ? simple_strtoul(s, &end, 16) : 0;
				if (s)
					s = (*end) ? end+1 : end;
			}
			env_ethaddr[7] = 0;

			at93c66_eeprom_write((const char *)env_ethaddr, ROM_MAC_SIZE, ROM_MAC_ADDR1);
			printf("Set MAC address1 from environment\n");
		}

		/* Read H/W Version from the U-Boot environment. */
		s = getenv("hwver");
		if(s == NULL) {
			printf ("No valid H/W Version in U-boot environment\n");
		} else {
			for (i = 0; i < 2; i++) {
				env_ver[i] = s ? simple_strtoul(s, &end, 16) : 0;
				if (s)
					s = (*end) ? end+1 : end;
			}

			at93c66_eeprom_write((const char *)env_ver, ROM_HW_SIZE,  ROM_HW_ADDR);
			printf("Set H/W Version(%2x.%02x)from environment\n", env_ver[0], env_ver[1]);
		}
	}
	else if (strcmp (argv[1], "write") == 0)
	{
		printf ("do not support now\n");
	}
	else if (strcmp (argv[1], "read") == 0)
	{
		if (strcmp (argv[2], "ethaddr") == 0) {
			at93c66_eeprom_read((char *)env_ethaddr, ROM_MAC_SIZE, ROM_MAC_ADDR0);
			printf("%02x:%02x:%02x:%02x:%02x:%02x\n",
				env_ethaddr[1], env_ethaddr[2], env_ethaddr[3],
				env_ethaddr[4], env_ethaddr[5], env_ethaddr[6]);
		}
		else if (strcmp (argv[2], "eth1addr") == 0) {
			at93c66_eeprom_read((char *)env_ethaddr, ROM_MAC_SIZE, ROM_MAC_ADDR1);
			printf("%02x:%02x:%02x:%02x:%02x:%02x\n",
				env_ethaddr[1], env_ethaddr[2], env_ethaddr[3],
				env_ethaddr[4], env_ethaddr[5], env_ethaddr[6]);
		}
		else if (strcmp (argv[2], "hwver") == 0) {
			at93c66_eeprom_read((char *)env_ethaddr, ROM_HW_SIZE, ROM_HW_ADDR);
			printf("%02x:%02x\n", env_ethaddr[0], env_ethaddr[1]);
		}
	}
	else if (strcmp (argv[1], "eral") == 0)
	{
		at93c66_erase_all();
		printf ("erase all\n");
	}
	else {
		goto usage;
	}

	return 0;

usage:
	cmd_usage(cmdtp);
	return 1;
}

U_BOOT_CMD(
	dvrrom, 3, 1,	do_save_dvrrom,
	"Saving environment to eeprom",
	"save - saveing ethaddr, hwver\n"
	"dvrrom read [ethaddr/eth1addr/hwver]\n"
	"dvrrom eral - erase all rom\n"
);
